/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz> (original developer)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include <nsISupports.h>

#ifndef _MOZCELLML_UNITS_H
#define _MOZCELLML_UNITS_H

class mozCellML;

class mozCellMLUnits
  : public nsISupports
{
public:
  NS_DECL_ISUPPORTS

  mozCellMLUnits(PRUint32 imp, const nsAString& comp, nsIDOMElement* tag);
  // base SI units...
  mozCellMLUnits();
  // builtin derivatives...
  mozCellMLUnits(mozCellMLUnits*, double exp, double offs=0.0, double mup=1.0);
  mozCellMLUnits(mozCellMLUnits*, double, mozCellMLUnits*, double);
  mozCellMLUnits(mozCellMLUnits*, double, mozCellMLUnits*, double,
                 mozCellMLUnits*, double);
  mozCellMLUnits(mozCellMLUnits*, double, mozCellMLUnits*, double,
                 mozCellMLUnits*, double, mozCellMLUnits*, double);

  nsresult Initialise(mozCellML* model,
                      nsDataHashtable<nsISupportsHashKey,
                                      PRUint32>&
                        importNumbers,
                      nsInterfaceHashtable<nsStringHashKey,
                                           mozCellMLUnits>&
                        allUnits
                     );
  
  PRBool IsSeen() { return seen; }
  PRBool IsDone() { return done; }
  void SetSeen() { seen = PR_TRUE; }
  void SetDone() { done = PR_TRUE; }
  double GetOffset() { return offset; }
  double GetMultiplier() { return multiplier; }

  // true if exactly the same(same units, same multiplier/offset)
  PRBool Equals(mozCellMLUnits* cf);
  // true if same units but possibly different multiplier/offset
  PRBool Equivalent(mozCellMLUnits* cf);

  PRBool GetIsBase() { return isbase; }
  void GetFQName(nsAString& fqname);
  static double PrefixToMultiplier(nsAString& prefix);

  /* This stuff should be private, workaround MSVC98 bug. */
  class UnitBase
    : public nsISupports
  {
  public:
    NS_DECL_ISUPPORTS

    UnitBase(nsCOMPtr<mozCellMLUnits> ibase, double iexponent)
      : base(ibase), exponent(iexponent) {};

    nsCOMPtr<mozCellMLUnits> base;
    double exponent;
  };

  PRUint32 GetUnitCount() { return unitList.Count(); };
  UnitBase* GetUnit(PRUint32 i) { return unitList[i]; };
private:

  PRBool isbase, seen, done;
  PRUint32 importNo;
  nsString component, name;
  double multiplier, offset;
  nsCOMArray<UnitBase> unitList;
  nsCOMPtr<nsIDOMElement> tag;

  void SwapBases(PRUint32 idx1, PRUint32 idx2);
  PRBool CompareBases(UnitBase* b1, UnitBase* b2);
  void SortBases(PRUint32 lower, PRUint32 upper);
};

#endif // _MOZCELLML_UNITS_H
