/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz> (original developer)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

int
main(int argc, char** argv)
{
  FILE* f;
  u_int32_t tmp32;
  u_int16_t tmp16;

  if (argc < 2)
  {
    printf("Usage: make_pe_noconsole filename\n");
    return -1;
  }
  f = fopen(argv[1], "r+");
  if (f == NULL)
  {
    printf("Couldn't open PE file.\n");
    return -1;
  }
  if (fread(&tmp16, 2, 1, f) != 1)
  {
    printf("Couldn't read file beginning.\n");
    return -1;
  }
  /* This only works on little endian, but that isn't a problem here... */
  if (tmp16 != (('M') | (('Z') << 8)))
  {
    printf("Not a windows executable.\n");
    return -1;
  }
  if (fseek(f, 0x3C, SEEK_SET) != 0)
  {
    printf("Seek failed.\n");
    return -1;
  }
  if (fread(&tmp32, 4, 1, f) != 1)
  {
    printf("Couldn't read new header offset.\n");
    return -1;
  }
  /* Overflow isn't a security risk because fseek will check everything... */
  tmp32 += 92;
  if (fseek(f, tmp32, SEEK_SET) != 0)
  {
    printf("Seek failed.\n");
    return -1;
  }
  if (fread(&tmp16, 2, 1, f) != 1)
  {
    printf("Read old value failed.\n");
    return -1;
  }
  if (fseek(f, tmp32, SEEK_SET) != 0)
  {
    printf("Seek failed.\n");
    return -1;
  }
  printf("Subsystem currently is %u\n", tmp16);
  tmp16 = 2;
  if (fwrite(&tmp16, 2, 1, f) != 1)
  {
    printf("Write new value failed.\n");
    return -1;
  }
}
